//------------------------------------
//Digitln hodiny s teplomrem
//MCU: AT Mega 8 (16MHz)
//RTC: PCF8563 (I2C $1)
//Teplotn idlo: LM75 (I2C $0)
//UART<->RS2323: MAX232CWE
//LCD:
//---adi HD44780
//---Zapojen:
//------RS: PORTB.5
//------RD: PORTB.4
//------EN: PORTB.3
//------D4: PORTB.2
//------D5: PORTB.1
//------D6: PORTB.0
//------D7: PORTD.7
//------------------------------------
//Autor: Ji ech
//Datum: 1/2015
//Verze: 1.0
//------------------------------------
#include <mega8.h>
#include <alcd.h>
#include <ctype.h>
#include <stdio.h>
#include <i2c.h>
#include <stdlib.h>
#include <delay.h>

//Verze
#define SWV 1.0
#define HWV 1.0
//Definice konstant T/F
#define TRUE 1
#define FALSE 0

//Definice konstant tlatek
#define DELAY_SHORT 20 //Doba ekn na druh vzorek pi men krtkho asu sepnut tlatka [ms]
#define DELAY_LONG 1200 //Doba ekn na druh vzorek pi men dlouhho asu sepnut tlatka [ms]
#define HLIMIT_SET_SEL 9 //Horn limit potu nastavovanch hodnot (hodnota nvratu zpt na zatek menu)

//Definice pozinch znaek menu
#define POSITION_SIG_TOUP "^^" //as/datum
#define POSITION_SIG_WKD "<<" //Den v tdnu

//Datum:
struct{
unsigned char day; //Den
unsigned char weekday; //Den v tdnu (nedle=0)
unsigned char month; //Msc
unsigned int year;  //Rok ve formtu 'RR'
}date;

//as:
struct{
unsigned char hour; //Hodiny
unsigned char minute; //Minuty
unsigned char second; //Sekundy
unsigned char temptimer; //asova vzorkovn teploty
}time;

//Teplota:
struct{
unsigned char temp_msb;
unsigned char temp_lsb;
unsigned int temp_data;
}temperature;

//Promnn konvertovan do ASCII
//Datum:
struct{
unsigned char dayc[6]; //ASCII den
unsigned char monthc[6]; //ASCI msc
unsigned char yearc[6]; //ASCII rok ve formtu 'RR'
}datec;

//Pole s nzvy dn (s diakritikou)
flash char weekdayc[7][7]={{'n','e','d',1,'l','e',' '},{'p','o','n','d',1,'l',2},{4,'t','e','r',3,' ',' '},{'s','t',5,'e','d','a',' '},{6,'t','v','r','t','e','k'},{'p',7,'t','e','k',' ',' '},{'s','o','b','o','t','a',' '}};

//as:
struct{
unsigned char hourc[6]; //ASCII hodiny
unsigned char minutec[6]; //ASCII minuty
unsigned char secondc[6]; //ASCII sekundy
}timec;

//Teplota:
struct{
unsigned char fullc[6];
}temperaturec;

bit vl; //Integrita dat RTC. 1:Error; 0:OK;
bit disp_erase_f; //Pznak nutnosti smazn hlky o stavu RTC z LCD.
eeprom unsigned char correction_sign; //Uruje, zda se pi kadodenn korekci (v 03:00:30) bude nastaven hodnota >correction< odetat:1 i pitat:0.
eeprom unsigned int correction; //Poet sekund pitanch/ odtanch pi denn korekci.
unsigned int corrections; //Doasn daj o konstant kompenztoru natan ze sriov linky, kter je uloen v pamti sram.
unsigned int correctionstmp; //Doasn daj o konstant kompenztoru pro ely konverze.
unsigned char correctionc[6]; //loit pro konstantu konvertovanou do ASCII.
bit first_rtc_data; //Byly nateny prvn asov informace od resetu mikrokontrolru:1; Po resetu:0.
bit afterreset=TRUE; //Mikrokontrolr je po resetu

//Tlatka:
unsigned char button_counts; //Poet cca 10ms cykl, po kter je dan tlatko zapnuto
unsigned char set_sel; //Vbr nastavovan hodnoty v menu (0: menu oputno)
unsigned char hlimit; //Horn limit nastavovanch hodnot
unsigned char llimit; //Spodn limit nastavovanch hodnot
unsigned char tmp_value[HLIMIT_SET_SEL]; //Pole doasnch hodnot (0:hodiny; 1:minuty; 2:sekundy; 3:dny; 4:msce; 5:roky; 6:dny v tdnu)
unsigned char c; //Promnn, do kter se nataj znaky ten z rozhran UART.
//------------------------------------

//Nastaven SDA/SCL i2c rozhran
#asm
    .equ __i2c_port=0x15 //Adresa portu C
    .equ __sda_bit=4
    .equ __scl_bit=5
#endasm


//------------------------------------

//Pevod do BCD: jestlie je vstup vt, ne 99, vrac 0xFF
unsigned char tobcd(unsigned char num){
unsigned char numh;
unsigned char numl;

if(num>99){
return 0xFF;
}

numh=num/10;
numl=num%10;
num=numh<<4;
num+=numl;
return num;
}

//Pevod z BCD: jestlie je alespo jedno 4bit slovo vt, ne 9, vrac 0xFF
unsigned char frombcd(unsigned char bcd){
unsigned char bcdh;
unsigned char bcdl;

bcdh=bcd>>4;
bcdl=bcd&0b00001111;
if(bcdh>9 || bcdl>9){
return 0xFF;
}
return (10*bcdh)+bcdl;
}

//Sekvenn ten dat z teplotnho idla a konverze
void temperature_read(void){
//Sekvenn ten dat z teplotnho idla
i2c_start();
i2c_write(0x90);
i2c_write(0x00);
i2c_start();
i2c_write(0x91);
temperature.temp_msb=i2c_read(1);
temperature.temp_lsb=i2c_read(0);
i2c_stop();

//Konverze dat o teplot
temperature.temp_data=temperature.temp_msb&0b01111111;
temperature.temp_msb>>=7;
temperature.temp_lsb>>=7;
itoa(temperature.temp_data,temperaturec.fullc);

}

//Zapsat data do RTC
void rtc_write(unsigned char address_write, unsigned char data_write){
i2c_start();
i2c_write(0xA2);
i2c_write(address_write);
i2c_write(data_write);
i2c_stop();
}

//st data z RTC
unsigned char rtc_read(unsigned char address_read){
unsigned char data_read;
i2c_start();
i2c_write(0xA2);
i2c_write(address_read);
i2c_start();
i2c_write(0xA3);
data_read=i2c_read(0);
i2c_stop();
return data_read;
}

//Inicializovat RTC
void rtc_init(void){
i2c_start();
i2c_write(0xA2);
i2c_write(0x0);
i2c_write(0b00001000); //CSR1: Test: Deaktivovn; POR: Aktivovn
i2c_write(0b00000000); //CSR2: TI_TP: Deaktivovno; AI:Deaktivovno; TI:Deaktivovno;
i2c_start();
i2c_write(0xA2);
i2c_write(0xD);
i2c_write(0b10000011); //CLKOUT CTRL: CLKOUT: aktivovn; frekvence: 1Hz;
i2c_write(0b00000000); //TIM CTRL: asova: deaktivovn;
i2c_stop();
}

//Sekvenn ten asu a data z RTC
void seq_read_cnv(void){
i2c_start();
i2c_write(0xA2);
i2c_write(0x2);
i2c_start();
i2c_write(0xA3);
time.second=i2c_read(1);
time.minute=i2c_read(1);
time.hour=i2c_read(1);
date.day=i2c_read(1);
date.weekday=i2c_read(1);
date.month=i2c_read(1);
date.year=i2c_read(1);
i2c_stop();

//prava a konverze dat o ase z kdu BCD do BIN
vl=time.second>>7;
time.second=frombcd(time.second&0b01111111);
time.minute=frombcd(time.minute&0b01111111);
time.hour=frombcd(time.hour&0b00111111);
date.day=frombcd(date.day&0b00111111);
date.weekday&=0b00000111;
date.month=frombcd(date.month&0b00011111);
date.year=frombcd(date.year);
}

//Uloit nastaven hodin z menu do RTC nebo MCU ((1:hodiny; 2:minuty; 3:sekundy; 4:dny; 5:msce; 6:roky; 7:dny v tdnu; 8:konstanta kompenztoru asov odchylky))
void val_save(unsigned char set_sel_scan){
switch(set_sel_scan){

    case 1:
    rtc_write(0x4,tobcd(tmp_value[set_sel_scan-1])); //Hodiny
    break;

    case 2:
    rtc_write(0x3,tobcd(tmp_value[set_sel_scan-1])); //Minuty
    break;

    case 3:
    rtc_write(0x2,tobcd(tmp_value[set_sel_scan-1])); //Sekundy
    break;

    case 4:
    rtc_write(0x5,tobcd(tmp_value[set_sel_scan-1])); //Dny
    break;

    case 5:
    rtc_write(0x7,tobcd(tmp_value[set_sel_scan-1])); //Msce
    break;

    case 6:
    rtc_write(0x8,tobcd(tmp_value[set_sel_scan-1])); //Roky
    break;

    case 7:
    rtc_write(0x6,tmp_value[set_sel_scan-1]); //Dny v tdnu
    break;

    case 8:
    correction=tmp_value[set_sel_scan-1];
}
}

//Vypsat den v tdnu
void display_weekday(void){
unsigned char i; //Itertor vpisu znak dn v tdnu
//Pokud je zpracovvno menu, vyp se pozin znaky pro dny v tdnu
if(set_sel!=0){
lcd_clear();
lcd_gotoxy(9,0);
lcd_putsf(POSITION_SIG_WKD);
lcd_gotoxy(9,1);
lcd_putsf("den"); //Vpis informace o nastavovanm parametru
lcd_gotoxy(0,0); //Den v tdnu bude vypisovn na prvn dek
}else{ //Pokud nen zpracovvno menu, den v tdnu je vypisovn na druh dek
lcd_gotoxy(9,1);
}
for(i=0;i!=7;i++){
lcd_putchar(weekdayc[date.weekday][i]);
}
}

//Zobrazit datum na LCD display:
void display_date(void){
   bit y; //Souadnice vpisu data (osa 'y')
   //Pokud je zpracovvno menu, smae se displej a vyp se pozin znaky.
   if(set_sel!=0){
   lcd_clear();
   y=1; //Vpis pozinch znak probhne na spodn dek
   switch(set_sel){
   case 4:
   lcd_gotoxy(0,y);
   break;

   case 5:
   lcd_gotoxy(3,y);
   break;

   case 6:
   lcd_gotoxy(6,y);
   break;
   }
   lcd_putsf(POSITION_SIG_TOUP);
   lcd_gotoxy(9,1);
   lcd_putsf("datum"); //Vpis informace o nastavovanm parametru
   y=0; //Vpis data probhne na horn dek
   }else{
   y=1; //Vpis data probhne na spodn dek (situace, kdy nen zpracovvno menu)
   }
   //Vpis data:
   lcd_gotoxy(0,y);
   itoa(date.day, datec.dayc);
   if(date.day<10){
   lcd_putchar('0');
   lcd_gotoxy(1,y);
   }
   lcd_puts(datec.dayc);

   lcd_gotoxy(3,y);
   itoa(date.month, datec.monthc);
   if(date.month<10){
   lcd_putchar('0');
   lcd_gotoxy(4,y);
   }
   lcd_puts(datec.monthc);

   lcd_gotoxy(6,y);
   itoa(date.year, datec.yearc);
   if(date.year<10){
   lcd_putchar('0');
   lcd_gotoxy(7,y);
   }
   lcd_puts(datec.yearc);


}

//Zobrazit as na LCD display:
void display_time(void){
   //Pokud je zpracovvno menu, smae se displej a vyp se pozin znaky.
   if(set_sel!=0){
   lcd_clear();
   switch(set_sel){
   case 1:
   lcd_gotoxy(0,1);
   break;

   case 2:
   lcd_gotoxy(3,1);
   break;

   case 3:
   lcd_gotoxy(6,1);
   break;
   }
   lcd_putsf(POSITION_SIG_TOUP);
   lcd_gotoxy(9,1);
   lcd_putchar(0x6); //Vpis informace o nastavovanm parametru (as)
   lcd_putsf("as");
   }
   lcd_gotoxy(0,0);
   itoa(time.hour, timec.hourc);
   if(time.hour<10){
   lcd_putchar('0');
   lcd_gotoxy(1,0);
   }
   lcd_puts(timec.hourc);

   lcd_gotoxy(3,0);
   itoa(time.minute, timec.minutec);
   if(time.minute<10){
   lcd_putchar('0');
   lcd_gotoxy(4,0);
   }
   lcd_puts(timec.minutec);

   lcd_gotoxy(6,0);
   itoa(time.second, timec.secondc);
   if(time.second<10){
   lcd_putchar('0');
   lcd_gotoxy(7,0);
   }
   lcd_puts(timec.secondc);




}

//Vypsat statick text asu na LCD
void display_static_time(void){
lcd_gotoxy(2,0);
lcd_putchar(':');
lcd_gotoxy(5,0);
lcd_putchar(':');
}
//Vypsat statick text data na LCD
void display_static_date(void){
bit y; //Souadnice statickho textu data (osa 'y')
//Pokud se zpracovv menu, statick text data se vypisuje na horn dek
if(set_sel!=0){
y=0;
}else{ //pokud se menu nezpracovv, statick text data se vypisuje na spodn dek
y=1;
}
lcd_gotoxy(2,y);
lcd_putchar('.');
lcd_gotoxy(5,y);
lcd_putchar('.');
}
//Vypsat statick text na LCD
void display_static(void){
display_static_time();
display_static_date();
lcd_gotoxy(8,0);
lcd_putchar('|');
lcd_gotoxy(8,1);
lcd_putchar('|');
lcd_gotoxy(14,0);
lcd_putchar(0);
lcd_gotoxy(15,0);
lcd_putchar('C');
lcd_gotoxy(12,0);
lcd_putchar(',');
}



//Defaultn nastaven RTC
void rtc_default(void){
if(vl==TRUE){//Jestlie je integrita dat v RTC naruena...
//Nastaven defaultnho asu po POR...
//00:00:00
rtc_write(0x3, 0b00000000);
rtc_write(0x4, 0b00000000);

//01.01.15>>tvrtek
rtc_write(0x5, 0b00000001);
rtc_write(0x7, 0b00000001);
rtc_write(0x8, 0b00010101);
rtc_write(0x6, 0b00000100);
}
}

//Servisn menu
void service_menu(void){
unsigned char address_write_servicemenu;//Adresa registru RTC kam se bude realizovat zpis
unsigned char hlimit; //Maximln platn hodnota vstupu od uivatele
bit llimit=FALSE; //Minimln platn hodnota vstupu od uivatele (1:1; 0:0)
bit continue_servicemenu=TRUE; //Opakovn vstupu do servisnho menu (1: aktivn; 0:neaktivn)

if(vl==TRUE){
rtc_default(); //defaultn nastaven RTC
rtc_write(0x2, 0b00000000);  //Nastaven sekund a VL na FALSE
}

display_static();

while(continue_servicemenu!=FALSE){
putsf("\n\nNastaven RTC ...\n>h:hodiny\n>m:minuty\n>s:sekundy\n>d:dny\n>w:dny v tdnu\n>e:msce\n>y:roky\n\nr>Vypsat data z RTC a teplotnho idla\nc>Nastaven denn kompenzace asov odchylky");
switch(getchar()){
    case 'h':
    printf(">Hodiny:[HH]\n>_ ");
    address_write_servicemenu=0x4;
    hlimit=23;
    break;

    case 'm':
    printf(">Minuty:[MM]\n>_ ");
    address_write_servicemenu=0x3;
    hlimit=59;
    break;

    case 's':
    printf(">Sekundy:[SS]\n>_ ");
    address_write_servicemenu=0x2;
    hlimit=59;
    break;

    case 'd':
    printf(">Den:[DD]\n>_ ");
    address_write_servicemenu=0x5;
    llimit=TRUE;
    hlimit=31;
    break;

    case 'w':
    printf(">Den v tdnu:[Ne:0;Po:1;t:2;St:3;t:4;P:5;So:6;]\n>_ ");
    address_write_servicemenu=0x6;
    hlimit=6;
    break;

    case 'e':
    printf(">Msc:[MM]\n>_ ");
    address_write_servicemenu=0x7;
    llimit=TRUE;
    hlimit=12;
    break;

    case 'y':
    printf(">Rok:[RR]\n>_ ");
    address_write_servicemenu=0x8;
    hlimit=99;
    break;

    case 'r':
    putsf("\nVpis dat z RTC a teplotnho idla:");
    putsf("\n>");
    rtc_init(); //Inicializace RTC (Nen jist, zda obvod byl i nebyl ji inicializovn)
    seq_read_cnv();  //Sekvenn ten dat z RTC (Datum a as) a nsledn konverze
    temperature_read(); //ten a konverze dat o teplot
    printf("%d:%d:%d\n%d.%d.%d\n", time.hour, time.minute, time.second, date.day, date.month, date.year);

    switch(date.weekday){
    case 0:
    putsf("nedle");
    break;

    case 1:
    putsf("pondl");
    break;

    case 2:
    putsf("ter");
    break;

    case 3:
    putsf("steda");
    break;

    case 4:
    putsf("tvrtek");
    break;

    case 5:
    putsf("ptek");
    break;

    case 6:
    putsf("sobota");
    break;

    default:
    putsf("chyba");
    }

    printf("\nData o ase ");
    if(vl==1){
    printf("ne");
    }
    printf("jsou relevantn.");
    address_write_servicemenu=0xFE; //Identifikace proveden pkazu bez zpisu do RTC.

    //Vpis dat o teplot...
    printf("\nTeplota: ");
    if(temperature.temp_msb==0){
    printf("%c", '+');
    }else{
    printf("%c", '-');
    }

    //Vpis desetinn sti dat o teplot...
    printf("%s,", temperaturec.fullc);
    if(temperature.temp_lsb==1){
    printf("%c", '5');
    }else{
    printf("%c", '0');
    }
    putsf("C\n>");
    break;

    case 'c':
    putsf("\nNastaven denn korekce asu v 03:00:30 \np>+/-\nv>Konstanta [s/d]\nr>Vypsat konfiguraci kompenzace asov odchylky\n>_ ");
    switch(getchar()){
    case 'p':
    putsf("\n1>Odtat\n0>Pitat\n>_ ");
    switch(getchar()){
    case '1':
    correction_sign=1;
    address_write_servicemenu=0xFE; //Identifikace proveden pkazu bez zpisu do RTC.
    break;

    case '0':
    correction_sign=0;
    address_write_servicemenu=0xFE; //Identifikace proveden pkazu bez zpisu do RTC.
    break;

    default:
    putsf("!_>Neplatn pkaz");
    address_write_servicemenu=0xFE; //Identifikace proveden pkazu bez zpisu do RTC.
    }
    break;

    case 'v':
    printf("\nNastavte konstantu kompenztoru v rozsahu 0..29.[xx]\n>_ ");
    scanf("%1u", &corrections);
    scanf("%1u", &correctionstmp);
    corrections*=10;
    corrections+=correctionstmp;
    if(corrections>29){
    correction=0;
    putsf("\nNeplatn rozsah vstupnch dat! Konstanta=0");
    }else{
    correction=corrections; //Uloen konstanty kompenztoru do intern pamti EEPROM
    }
    address_write_servicemenu=0xFE; //Identifikace proveden pkazu bez zpisu do RTC.
    break;

    case 'r':
    putsf("Vypsat konfiguraci kompenzace asov odchylky...\n");
    if(correction_sign==1){
    putchar('-');
    }else{
    putchar('+');
    }
    printf("%u", correction);
    break;

    default:
    putsf("!_>Neplatn pkaz");
    address_write_servicemenu=0xFE; //Identifikace proveden pkazu bez zpisu do RTC.
    }
    break;

    default:
    putsf("!_>Neplatn pkaz");
    address_write_servicemenu=0xFE; //Identifikace proveden pkazu bez zpisu do RTC.
    break;
}

    //Oven, zda byl zadn platn pkaz
    if(address_write_servicemenu!=0xFF){
    if(address_write_servicemenu!=0xFE){
    //Scannovn vstupnch dat a jejich nsledn konverze...
    scanf("%2u", &c);

    if(!((c<llimit && c>hlimit))){  //Oven limitu
    printf("\n_>Volba:%u\n",c);
    rtc_write(address_write_servicemenu,tobcd(c));
    putsf("_>Data uloena\nPro ukonen servisnho menu stisknte 'x' ...");
    }else{
    putsf("!_>Neplatn formt vstupu. Data nebyla uloena!.");
    }
    delay_ms(1500);
    }
    if(UDR=='x'){
    continue_servicemenu=FALSE;
    putsf("_>Pokraovn bhu programu");
    }else{
    continue_servicemenu=TRUE;
    }
    }else{
    putsf("!_>Formt vstupu je neplatn");
    }
    }
}



//Extern peruen INT0: Nbn hrana; "CLKOUT"
interrupt [EXT_INT0] void ext_int0_isr(void){

PORTC.0=~PORTC.0;
PORTC.1=~PORTC.1;

//Sekvenn ten dat o ase a jejich konverze
seq_read_cnv();

//V 03:00:30 hodin spustit denn kompenzaci asov odchylky
if(time.hour==3 && time.minute==0 && time.second==30){
if(correction_sign==1){
time.second-=correction;
}else{
time.second+=correction;
rtc_write(0x2,(0b01111111&tobcd(time.second)));
}
}

//Sekvenn ten dat z teplotnho idla
temperature_read();

//Vpis znaku kladnosti/zpornosti teploty
lcd_gotoxy(9,0);
if(temperature.temp_msb==FALSE){
lcd_putchar('+');
}else{
lcd_putchar('-');
}

//Vpis cel sti teploty
lcd_gotoxy(10,0);
if(temperature.temp_data<10){
lcd_putchar('0');
lcd_gotoxy(11,0);
}
lcd_puts(temperaturec.fullc);

//Vpis polovinov sti teploty
lcd_gotoxy(13,0);
if(temperature.temp_lsb==TRUE){
lcd_putchar('5');
}else{
lcd_putchar('0');
}

//Vypsat data o ase na LCD
    //Pokud byla vypsna chybov hlka RTC, kter ji neplat, smazat displej a vypsat statick text
if(disp_erase_f==TRUE && vl==TRUE){
lcd_clear();
display_static();
disp_erase_f=FALSE;
}
if(vl==FALSE){
display_time();
display_date();
display_weekday();
}else{
//Signalizace nerelevantnch dat RTC
disp_erase_f=TRUE;
lcd_gotoxy(2,0);
lcd_putsf("Nastavte RTC  ");
lcd_gotoxy(0,1);
lcd_putsf(">_ ... ... ...");
delay_ms(500);
}

first_rtc_data=TRUE; //Byly nateny prvn asov informace od resetu mikrokontrolru
}

//Extern peruen INT1: Sestupn hrana; "ALARM"
interrupt [EXT_INT1] void ext_int1_isr(void){
}

void main(void){

//Nastaven vstupnch pin
DDRC=0b00000011;
PORTC=0b00000001;


//Nastaven rozhran UART
UCSRA=0x00;
UCSRB=0x18;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x67;

//Vypnut analogovho kompartoru
ACSR=0b10000000;

//Inicializace LCD
lcd_init(16);

//definice znaku $0; ''
lcd_write_byte(0b01000000,0b00000100);
lcd_write_byte(0b01000001,0b00001010);
lcd_write_byte(0b01000010,0b00000100);
lcd_write_byte(0b01000011,0b00000000);
lcd_write_byte(0b01000100,0b00000000);
lcd_write_byte(0b01000101,0b00000000);
lcd_write_byte(0b01000110,0b00000000);
lcd_write_byte(0b01000111,0b00000000);

//definice znaku $1; ''
lcd_write_byte(0b01001000,0b00001010);
lcd_write_byte(0b01001001,0b00000100);
lcd_write_byte(0b01001010,0b00001110);
lcd_write_byte(0b01001011,0b00010001);
lcd_write_byte(0b01001100,0b00011111);
lcd_write_byte(0b01001101,0b00010000);
lcd_write_byte(0b01001110,0b00001111);
lcd_write_byte(0b01001111,0b00000000);

//definice znaku $2; ''
lcd_write_byte(0b01010000,0b00000010);
lcd_write_byte(0b01010001,0b00000100);
lcd_write_byte(0b01010010,0b00001100);
lcd_write_byte(0b01010011,0b00000100);
lcd_write_byte(0b01010100,0b00000100);
lcd_write_byte(0b01010101,0b00000100);
lcd_write_byte(0b01010110,0b00001110);
lcd_write_byte(0b01010111,0b00000000);

//definice znaku $3; ''
lcd_write_byte(0b01011000,0b00000010);
lcd_write_byte(0b01011001,0b00000100);
lcd_write_byte(0b01011010,0b00010001);
lcd_write_byte(0b01011011,0b00010001);
lcd_write_byte(0b01011100,0b00001111);
lcd_write_byte(0b01011101,0b00000001);
lcd_write_byte(0b01011110,0b00001110);
lcd_write_byte(0b01011111,0b00000000);

//definice znaku $4; ''
lcd_write_byte(0b01100000,0b00000010);
lcd_write_byte(0b01100001,0b00000100);
lcd_write_byte(0b01100010,0b00010001);
lcd_write_byte(0b01100011,0b00010001);
lcd_write_byte(0b01100100,0b00010001);
lcd_write_byte(0b01100101,0b00010011);
lcd_write_byte(0b01100110,0b00001101);
lcd_write_byte(0b01100111,0b00000000);

//definice znaku $5; ''
lcd_write_byte(0b01101000,0b00001010);
lcd_write_byte(0b01101001,0b00000100);
lcd_write_byte(0b01101010,0b00010110);
lcd_write_byte(0b01101011,0b00011001);
lcd_write_byte(0b01101100,0b00010000);
lcd_write_byte(0b01101101,0b00010000);
lcd_write_byte(0b01101110,0b00010000);
lcd_write_byte(0b01101111,0b00000000);

//definice znaku $6; ''
lcd_write_byte(0b01110000,0b00001010);
lcd_write_byte(0b01110001,0b00000100);
lcd_write_byte(0b01110010,0b00001110);
lcd_write_byte(0b01110011,0b00010000);
lcd_write_byte(0b01110100,0b00010000);
lcd_write_byte(0b01110101,0b00010001);
lcd_write_byte(0b01110110,0b00001110);
lcd_write_byte(0b01110111,0b00000000);

//definice znaku $7; ''
lcd_write_byte(0b01111000,0b00000010);
lcd_write_byte(0b01111001,0b00000100);
lcd_write_byte(0b01111010,0b00001110);
lcd_write_byte(0b01111011,0b00000001);
lcd_write_byte(0b01111100,0b00001111);
lcd_write_byte(0b01111101,0b00010001);
lcd_write_byte(0b01111110,0b00001111);
lcd_write_byte(0b01111111,0b00000000);


//Vpis statickho textu na LCD
display_static();

//Inicializace RTC
rtc_init();

//Naten dat z RTC
seq_read_cnv();

//Oven, zda jsou data v RTC relevantn
if(vl==TRUE){
//Defaultn nastaven RTC
rtc_default();
vl=TRUE;
}

//Extern peruen INT0: nbn hrana; INT1: sestupn hrana
GICR|=0b11000000;
MCUCR=0b00001011;

//Kontrola korektnosti konstanty kompenztoru asov odchylky
if(correction>29){
correction=0;
}

//Vstup do servisnho menu pstroje
putsf("Digitln hodiny s teplomrem v. 1.0\n          3/15' Ji ech");
putsf("m>vstup do servisnho menu\nx>opustit servisn menu");
if(PIND.5==TRUE){
delay_ms(20);
if(PIND.5==TRUE){
service_menu();
}
}else{
putsf("_>Pokraovn bhu programu ...\n");
}

//Globln povolen peruen
#asm
sei
#endasm



//Hlavn smyka programu
while(1){
//Kontrola integrity dat RTC
if(vl==TRUE && first_rtc_data==TRUE && afterreset==TRUE){
putsf("\nIntegrita dat RTC naruena. Nastavte RTC!");
afterreset=FALSE;
}
//Scannovn stisknut klvesy 'm' a v ppad jejho stisku vyvolat servisn menu
if(UDR=='m'){
service_menu();
}
//Snmn tlatka '1'
//Oeten zkmit
if(PIND.5==TRUE){
delay_ms(DELAY_SHORT);
if(PIND.5==TRUE){
//Potn doby stisku
while(PIND.5==TRUE){
//Oeten monho peteen unsigned char
if(button_counts!=0xFF){
delay_ms(10);
button_counts++;
}
}
//Vyhodnocovn doby stisku
if(button_counts>50){
button_counts=0;
//Pkazy pro akci po dlouhm stisknut
//Vstup do menu asu
rtc_default(); //defaultn nastaven RTC
rtc_write(0x2, 0b00000000);  //Nastaven sekund a VL na FALSE
rtc_write(0x0, 0b00100000); //Zastaven RTC
//Naten dat z RTC do pole s doasnmi daty:
tmp_value[0]=time.hour;
tmp_value[1]=time.minute;
tmp_value[2]=time.second;
tmp_value[3]=date.day;
tmp_value[4]=date.month;
tmp_value[5]=date.year;
tmp_value[6]=date.weekday;
set_sel=1;//Zan se nastavenm hodin
display_time(); //Vpis asu (Ostatn se smae vlivem aktivnho menu)
display_static_time(); //Vpis statickho textu asu
hlimit=23;
llimit=0;
//------------------------------------------------
while(set_sel!=FALSE){ //Dokud nen oputno menu
if(PIND.5==TRUE){
delay_ms(DELAY_SHORT);
if(PIND.5==TRUE){
//Potn doby stisku
while(PIND.5==TRUE){
//Oeten monho peteen unsigned char
if(button_counts!=0xFF){
delay_ms(10);
button_counts++;
}
}
//Vyhodnocovn doby stisku
if(button_counts<50){
button_counts=0;
//Pkazy pro akci po krtkm stisknut
//Dekrementace hodnoty
if(tmp_value[set_sel-1]!=llimit){ //Srovnvn hodnoty se spodnm limitem
tmp_value[set_sel-1]--; //Dekrementace
val_save(set_sel);//Uloen dat do RTC
seq_read_cnv(); //Sekvenn ten dat z RTC
if(set_sel<4){
    display_time(); //Vpis asu (Ostatn se smae vlivem aktivnho menu)
    display_static_time(); //Vpis statickho textu asu
}else if(set_sel<7){ //Pokud se nastavuje datum, vyp se pozin znaky data
    display_date(); //Vpis data (Ostatn se smae vlivem aktivnho menu)
    display_static_date(); //Vpis statickho textu data
}else if(set_sel==7){ //Pokud se nastavuje den v tdnu, vyp se pozin znaky dne v tdnu
display_weekday();
}else if(set_sel==8){ //Pokud se nastavuje kompenzace asov odchylky, vype se text.
lcd_clear();
lcd_gotoxy(0,0);
if(correction_sign==TRUE){ //Vpis kladnosti/ zpornosti konstanty kompenztoru asov odchylky
lcd_putchar('-');
}else{
lcd_putchar('+');
}

// Vpis hodnoty konstanty kompenztoru asov odchylky
itoa(correction, correctionc);
lcd_gotoxy(1,0);
lcd_puts(correctionc);
if(correction<10){
lcd_gotoxy(2,0);
}else{
lcd_gotoxy(3,0);
}
lcd_putchar('s');

lcd_gotoxy(0,1);
lcd_putsf("denn");
lcd_gotoxy(4,1);
lcd_putchar(2); //''
lcd_gotoxy(6,1);
lcd_putsf("kompenzace");
}else{
tmp_value[set_sel-1]=hlimit-1;
}
}else if(set_sel==8){ //Zmna kladnosti/ zpornosti konstanty kompenztoru asov odchylky
if(correction_sign==TRUE){
correction_sign=FALSE;
}else{
correction_sign=TRUE;
}
lcd_gotoxy(0,0);
if(correction_sign==TRUE){ //Vpis kladnosti/ zpornosti konstanty kompenztoru asov odchylky
lcd_putchar('-');
}else{
lcd_putchar('+');
}

if(correction<10){
lcd_gotoxy(2,0);
}else{
lcd_gotoxy(3,0);
}
lcd_putchar('s');
}
}else{
button_counts=0;
//Pkazy pro akci po dlouhm stisknut
//Skok na dal nastavovanou hodnotu
if(set_sel==(HLIMIT_SET_SEL-1)){ //Srovnn hodnoty s hornm limitem
set_sel=0; //Peskok na zatek menu (menu se ukon a v nsledujc podmnce nabyde promnn set_sel hodnoty 1 (hodiny)
}
if(set_sel!=HLIMIT_SET_SEL){ //Srovnvn hodnoty s hornm limitem
set_sel++;
switch(set_sel){ //Nastavovn limitu zadvanch hodnot
    case 1:
    //Hodiny
    hlimit=23;
    llimit=0;
    break;

    case 2:
    //Minuty:
    hlimit=59;
    //Spodn limit se nemn
    break;

    //Sekundy: limit se nemn

    case 4:
    //Dny:
    hlimit=31;
    llimit=1;
    break;

    case 5:
    //Msce:
    hlimit=12;
    //Spodn limit se nemn
    break;

    case 6:
    //Roky:
    hlimit=99;
    llimit=0;
    break;

    case 7:
    //Dny v tdnu:
    hlimit=6;
    //Spodn limit se nemn
    break;

    case 8:
    //Konstanta kompenztoru asov odchylky (kladnost/zapornost je nastavovna promnnou >correction_sign<)
    hlimit=29;
    //Spodn limit se nemn
    break;
}
//Pokud se nastavuje as, vyp se pozin znaky asu
if(set_sel<4){
    display_time(); //Vpis asu (Ostatn se smae vlivem aktivnho menu)
    display_static_time(); //Vpis statickho textu asu
}else if(set_sel<7){ //Pokud se nastavuje datum, vyp se pozin znaky data
    display_date(); //Vpis data (Ostatn se smae vlivem aktivnho menu)
    display_static_date(); //Vpis statickho textu data
}else if(set_sel==7){ //Pokud se nastavuje den v tdnu, vyp se pozin znaky dne v tdnu
display_weekday();
}else if(set_sel==8){ //Pokud se nastavuje kompenzace asov odchylky, vype se text.
lcd_clear();
lcd_gotoxy(0,0);
if(correction_sign==TRUE){ //Vpis kladnosti/ zpornosti konstanty kompenztoru asov odchylky
lcd_putchar('-');
}else{
lcd_putchar('+');
}
// Vpis hodnoty konstanty kompenztoru asov odchylky
itoa(correction, correctionc);
lcd_gotoxy(1,0);
lcd_puts(correctionc);

lcd_gotoxy(0,1);
lcd_putsf("denn");
lcd_gotoxy(4,1);
lcd_putchar(2); //''
lcd_gotoxy(6,1);
lcd_putsf("kompenzace");

if(correction<10){
lcd_gotoxy(2,0);
}else{
lcd_gotoxy(3,0);
}
lcd_putchar('s');
}
}
}
}
}
//------------------------------------------------
//------------------------------------------------
//Scannovn tlatka '2'
if(PIND.4==TRUE){
delay_ms(DELAY_SHORT);
if(PIND.4==TRUE){
//Potn doby stisku
while(PIND.4==TRUE){
//Oeten monho peteen unsigned char
if(button_counts!=0xFF){
delay_ms(10);
button_counts++;
}
}
//Vyhodnocovn doby stisku
if(button_counts<50){
button_counts=0;
//Pkazy pro akci po krtkm stisknut
//Inkrementace hodnoty
if(tmp_value[set_sel-1]!=hlimit){ //Srovnvn hodnoty s hornm limitem
tmp_value[set_sel-1]++; //Inkrementace
val_save(set_sel);//Uloen dat do RTC
seq_read_cnv(); //Sekvenn ten dat z RTC
if(set_sel<4){
    display_time(); //Vpis asu (Ostatn se smae vlivem aktivnho menu)
    display_static_time(); //Vpis statickho textu asu
}else if(set_sel<7){ //Pokud se nastavuje datum, vyp se pozin znaky data
    display_date(); //Vpis data (Ostatn se smae vlivem aktivnho menu)
    display_static_date(); //Vpis statickho textu data
}else if(set_sel==7){ //Pokud se nastavuje den v tdnu, vyp se pozin znaky dne v tdnu
display_weekday();
}else if(set_sel==8){ //Pokud se nastavuje kompenzace asov odchylky, vype se text.
lcd_clear();
lcd_gotoxy(0,0);
if(correction_sign==TRUE){ //Vpis kladnosti/ zpornosti konstanty kompenztoru asov odchylky
lcd_putchar('-');
}else{
lcd_putchar('+');
}

// Vpis hodnoty konstanty kompenztoru asov odchylky
itoa(correction, correctionc);
lcd_gotoxy(1,0);
lcd_puts(correctionc);

lcd_gotoxy(0,1);
lcd_putsf("denn");
lcd_gotoxy(4,1);
lcd_putchar(2); //''
lcd_gotoxy(6,1);
lcd_putsf("kompenzace");

if(correction<10){
lcd_gotoxy(2,0);
}else{
lcd_gotoxy(3,0);
}
lcd_putchar('s');

}else{
tmp_value[set_sel-1]=llimit+1;
}
}
}else{
button_counts=0;
//Pkazy pro akci po dlouhm stisknut
printf("\nOputn menu");
//Oputn menu bez uloen hodnot
rtc_write(0x0, 0b00000000); //Sputn RTC
set_sel=0; //Menu oputno
display_static(); //Vpis statickho textu
}

}
}
//------------------------------------------------
}
}
set_sel=0;
}
}
}
}
